function adrv
% AD = ADRV(MODEL,APPROX)
%
% This program copmutes analytical first and second (if approx=2)
% derivatives of the function f(yp,y,xp,x) with respect to x, y, xp, and yp.
%
% INPUT
%	MODEL	: model description, (.f,.x,.y,.xp,.yp)
%	APPROX	: approximation order, optional
%
% OUTPUT
%   AD		: analytical first and second derivatives of f.
%			   .f,.fx,.fxp,.fy,.fyp,
%              .fypyp,.fypy,.fypxp,.fypx,.fyyp,.fyy,.fyxp,.fyx,
%              .fxpyp,.fxpy,.fxpxp,.fxpx,.fxyp,.fxy,.fxxp,.fxx

% Sungbae An
% Updated: 08/16/2005 

load('.\An2007_2\model.mat');

% function and arguments
f  = dsge.f;
x  = dsge.x;
y  = dsge.y;
xp = dsge.xp;
yp = dsge.yp;

% call Stephanie-Grohe and Uribe procedure
[fx,fxp,fy,fyp,fypyp,fypy,fypxp,fypx,fyyp,fyy,fyxp,fyx,fxpyp,fxpy,fxpxp,fxpx,fxyp,fxy,fxxp,fxx] = anal_deriv(f,x,y,xp,yp,approx);

ad.f = matlabFunction(f);
ad.fx = matlabFunction(fx);
ad.fxp = matlabFunction(fxp);
ad.fy = matlabFunction(fy);
ad.fyp = matlabFunction(fyp);
ad.fypyp = matlabFunction(fypyp);
ad.fypy = matlabFunction(fypy);
ad.fypxp = matlabFunction(fypxp);
ad.fypx = matlabFunction(fypx);
ad.fyyp = matlabFunction(fyyp);
ad.fyy = matlabFunction(fyy);
ad.fyxp = matlabFunction(fyxp);
ad.fyx = matlabFunction(fyx);
ad.fxpyp = matlabFunction(fxpyp);
ad.fxpy = matlabFunction(fxpy);
ad.fxpxp = matlabFunction(fxpxp);
ad.fxpx = matlabFunction(fxpx);
ad.fxyp = matlabFunction(fxyp);
ad.fxy = matlabFunction(fxy);
ad.fxxp = matlabFunction(fxxp);
ad.fxx = matlabFunction(fxx);


save('.\An2007_2\model.mat','ad','-append');
end
    