function model
% new Keynesian DSGE model: monetary policy rules

% Sungbae An
% Created: 02/14/2005
% Updated: 08/11/2005

approx = 2;
% define symbolic parameters
syms tau nu phi psi1 psi2 rhor rhog rhoz sigg sigr sigz bet pist gst  % model parameters
	
% define symbolic variables
%
% time index:
%   0 yesterday
%   1 today
%   2 tomorrow
syms c1 c2 y1 y2 pi1 pi2 R0 R1 e1 e2 g1 g2 z1 z2 Rc1 Rc2 ys0 ys1

% system equations including Euler
% taking expectation on time 1 information, then we have zero.
f1  = 1 - exp(-tau*c2+tau*c1+R1-z2-pi2);
f2  = (1-nu)/nu/phi/(pist^2)*(exp(tau*c1)-1) - (exp(pi1)-1)*((1-1/2/nu)*exp(pi1)+1/2/nu) + bet*(exp(pi2)-1)*exp(-tau*c2+tau*c1+y2-y1+pi2);
f3  = exp(c1-y1) - exp(-g1) + phi*pist^2*gst/2*(exp(pi1)-1)^2;
f4  = R1 - rhor*R0 - (1-rhor)*(psi1*pi1 + psi2*(y1-g1)) + e1;  %y1-g1
f5  = e2;
f6  = g2 - rhog*g1;
f7  = z2 - rhoz*z1;
f8  = Rc1 - R1;
f9  = ys1 - y1;


% create function f
dsge.f = [f1;f2;f3;f4;f5;f6;f7;f8;f9];

% define the vector of controls, y, and states, x
dsge.x  = [ys0 R0 e1 g1 z1];
dsge.y  = [c1 y1 pi1 Rc1];
dsge.xp = [ys1 R1 e2 g2 z2];
dsge.yp = [c2 y2 pi2 Rc2];

% define model shock
eta = [0,0,0; 0,0,0; sigr,0,0; 0,sigg,0; 0,0,sigz];

dim.x1 = 2;
dim.x2 = 3;
dim.x = 5;
dim.z = 3;
dim.y = 3;
dim.u = 3;
dim.e = 3;

save('.\An2007_2\model.mat','dsge','eta','approx','dim');
end
