clc
clear all
path_m = '.\An2007_2\';    % Setting the model path
path(path_m,path);
path('tool',path);
path('.\tool\sgu\',path);

load([path_m,'model.mat']);

% simulation settings
para = [2,0.1,0.33,1.5,0.13,1.0,3.2,0.55,0.75,0.95,0.9,0.2,0.6,0.3];

set_r = [0.1,0.05,0.01,0.1];               % Table 1-3
set_np = [5e2,1e3,2e3,4e3,1e4,2e4,4e4];

n_rep = 100;
n_filt = 4;
n_np= 7;
n_tab = 4;
n_data = 10;
ll = nan(n_rep,n_filt,n_np,n_tab,n_data);
filename = '.\output\loglikevar.mat';
save(filename,'filename');

% initialization
[nd,ra,pia,gamq,neta,nsig] = ndrv(para,ad,eta,approx);
[pf,~] = polappx(nd,neta,approx);
sysm = sysmat(pf,ra,pia,gamq,neta,nsig);

% parallel computation
poolobj = parpool;

for i_data = 1:n_data
    data = simulate_apf(para,1e10);
    for i_tab = 1:n_tab-1
        r = set_r(i_tab);
        sysm.H = sqrt(r*diag(var(data))); 
        for i_np = 1:n_np
            np = set_np(i_np);
            parfor i_rep = 1:n_rep
                ll1(i_rep) = filter_bf(sysm,dim,data,np);
                ll2(i_rep) = filter_cpf(sysm,dim,data,np);
                ll3(i_rep) = filter_cdpf(sysm,dim,data,np);
                ll4(i_rep) = filter_apf(sysm,dim,data,np);      
            end
            ll(:,1,i_np,i_tab,i_data) = ll1;
            ll(:,2,i_np,i_tab,i_data) = ll2;
            ll(:,3,i_np,i_tab,i_data) = ll3;
            ll(:,4,i_np,i_tab,i_data) = ll4;  
            disp(num2str([i_np,i_tab,i_data]));
        end
    end
    save(filename,'ll');
end
% Table 4
i_tab = n_tab;
df = 15;
for i_data = 1:n_data
    data = simulate_apf(para,df);
    r = set_r(i_tab);
    sysm.H = sqrt(r*diag(var(data))); 
    for i_np = 1:n_np
        np = set_np(i_np);
        parfor i_rep = 1:n_rep
            ll1(i_rep) = filter_bf(sysm,dim,data,np);
            ll2(i_rep) = filter_cpf(sysm,dim,data,np);
            ll3(i_rep) = filter_cdpf(sysm,dim,data,np);
            ll4(i_rep) = filter_apf(sysm,dim,data,np);               
        end
        ll(:,1,i_np,i_tab,i_data) = ll1;
        ll(:,2,i_np,i_tab,i_data) = ll2;
        ll(:,3,i_np,i_tab,i_data) = ll3;
        ll(:,4,i_np,i_tab,i_data) = ll4;  
        disp(num2str([i_np,i_tab,i_data]));
    end
    save(filename,'ll');
end
delete(poolobj);

% calculating variance
ll_var = nan(n_filt,n_np,n_tab);
for i_tab = 1:n_tab
    for i_np = 1:n_np
        for i_filt = 1:n_filt
            ll_var(i_filt,i_np,i_tab) = mean(var(reshape(ll(:,i_filt,i_np,i_tab,:),[n_rep,n_data])));
        end
    end
 end
save(filename,'ll_var','-append');