function [loglik,weight] = filter_bf(sysm,dim,data,N)
% Evaluating log-likelihood function by Bootstrap filter

global ix1 ix2 
ix1 = 1:dim.x1;
ix2 = dim.x1+1:dim.x;

H2 = sysm.H*sysm.H';    % covariance of measurement errors

% initials
T = size(data,1);
loglik = 1:T;

% initial value of s at t=0
Sigma =  solve_cov(sysm,dim);
xt = zeros(dim.x,N);
xt(ix2,:) = chol(Sigma)'*randn(dim.u,N);

%ess = 1:T;
%Exf = nan(dim.x,T);
weight = nan(N,T);
for t = 1:T
    % sampling
    Exp = fun_h(sysm,xt);  
    e3 = sysm.R(ix2,:)*randn(dim.u,N);     
    xt(ix1,:) = Exp(ix1,:);          
    xt(ix2,:) = Exp(ix2,:) + e3;

    yt = data(t,:)';
    e2 = yt(:,ones(N,1)) - fun_g(sysm,xt);  % p(y_t|x_t)   
        
    lnwt = lmnpdf2(e2,H2);
    dfac = max(lnwt);
    wt   = exp(lnwt-dfac);
    loglik(t) = log(sum(wt)) - log(N) + dfac;
    
    weight(:,t) = wt;
   % cv = sum((wt-mean(wt)).^2)/(N-1)/mean(wt)^2;
   % ess(t) = N/(1+cv);
   % Exf(:,t) = xt*wt/sum(wt);
    
    xt = resmpl_mul(xt,wt/sum(wt));   
    
end
loglik = sum(loglik);
end


%--------------------------------------------------------------------------
function[G0] = solve_cov(sysm,dim)
global ix2
nx2 = dim.x2;
Sigmau = sysm.R(ix2,:)*sysm.R(ix2,:)';
G0v = (eye(nx2^2)-kron(sysm.hl(ix2,ix2),sysm.hl(ix2,ix2)))\Sigmau(1:nx2^2)';
G0  = zeros(nx2,nx2);
G0(1:nx2^2) = G0v;
end

function y = fun_h(sysm,x)
global ix1
N = size(x,2);
y = sysm.hc(:,ones(N,1)) + sysm.hl*x;
y(ix1,:) = y(ix1,:) + qf(sysm.hq,x);
end

function y = fun_g(sysm,x)
N = size(x,2);
z = sysm.gc(:,ones(N,1)) + sysm.gl*x + qf(sysm.gq,x); 
y = sysm.c(:,ones(N,1)) + sysm.zz*z + sysm.zx*x;
end

function y = qf(Q,x,s)
if nargin == 2
    s = x;
end
[nx,N] = size(x);
ny = size(Q,1)/nx;
a = Q*x;
b = a.*repmat(s,ny,1);
c = sum(reshape(b,nx,ny*N),1);
y = reshape(c,ny,N);
end

function [y,ind]= resmpl_mul(x,w)
% Y=RESMPL(X,W)
% resamples from X with weights W, with replacement
% based on multinomial sampling
N = size(x,2);
ind = randsample(1:N,N,true,w);
y = x(:,ind);
end

