function [pf,flag] = polappx(nd,eta,approx)
% PF = POLAPPX(ND,ETA,APPROX)
% approximates policy functions
%
% INPUT
%	ND     : numerical derivatives of f at the steady states
%   ETA    : error
%   APPROX : approximation order (default=2)
%
% OUTPUT
%	PF     : approximated policy function
%
% Call: gx_hx.m, gxx_hxx.m, and gss_hss.m (by Schmidtt-Groehe and Uribe)

% check input argument


%--------------------------------------------------------------------
% first order approximation
%--------------------------------------------------------------------
[pf.gx,pf.hx,flag] = gx_hx(nd.fy,nd.fx,nd.fyp,nd.fxp);

%--------------------------------------------------------------------
% second order approximation
%--------------------------------------------------------------------
if flag == 1 && approx == 2;
	% gxx_hxx.m
	[pf.gxx,pf.hxx] = gxx_hxx_noloop(nd.fx,nd.fxp,nd.fy,nd.fyp,nd.fypyp,nd.fypy,...
        nd.fypxp,nd.fypx,nd.fyyp,nd.fyy,nd.fyxp,nd.fyx,nd.fxpyp,nd.fxpy,...
        nd.fxpxp,nd.fxpx,nd.fxyp,nd.fxy,nd.fxxp,nd.fxx,pf.hx,pf.gx);


	% gss_hss.m
	[pf.gss,pf.hss] = gss_hss(nd.fx,nd.fxp,nd.fy,nd.fyp,nd.fypyp,nd.fypy,nd.fypxp,...
        nd.fypx,nd.fyyp,nd.fyy,nd.fyxp,nd.fyx,nd.fxpyp,nd.fxpy,nd.fxpxp,nd.fxpx,...
        nd.fxyp,nd.fxy,nd.fxxp,nd.fxx,pf.hx,pf.gx,pf.gxx,eta);
                                                    
else
    pf = 0;
    
end
end

